=head1 NAME

Tails::UDisks - role providing a connection to UDisks via DBus

=cut

package Tails::UDisks;
use Moose;

use 5.10.1;
use namespace::autoclean;
use warnings FATAL => 'all';
use autodie qw(:all);

use Carp::Assert::More;
use File::stat;
use List::MoreUtils qw{any};
use List::Util qw{first};
use Method::Signatures::Simple;
use Tails::Constants;
use Unix::Mknod qw(:all);

use Locale::gettext;
use POSIX;
setlocale(LC_MESSAGES, "");
textdomain("tails-perl-lib");

with 'Tails::Role::HasDBus::System';

has 'constants' =>
    is         => 'ro',
    isa        => 'Tails::Constants',
    lazy_build => 1,
    metaclass  => 'NoGetopt',
    handles    => [ qw{system_partition_label}];

has 'udisks_service' =>
    is         => 'ro',
    lazy_build => 1, # Let's decide the right initialization order in BUILD
    isa        => 'Net::DBus::RemoteService';

has 'udisks_object' =>
    is         => 'ro',
    lazy_build => 1, # Let's decide the right initialization order in BUILD
    isa        => 'Net::DBus::RemoteObject';

sub BUILD {
    my $self = shift;

    # Force initialization in correct order
    assert_defined($self->dbus);
    assert_defined($self->udisks_service);
    assert_defined($self->udisks_object);
}

method _build_constants {
    Tails::Constants->new();
}

method _build_udisks_service {
    $self->dbus->get_service("org.freedesktop.UDisks");
}

method _build_udisks_object {
    $self->udisks_service->get_object(
        "/org/freedesktop/UDisks",
        "org.freedesktop.UDisks"
    );
}

method get_device_property ($device, $property) {
    assert_defined($device);
    assert_defined($property);

    $self->udisks_service
        ->get_object($device)
        ->as_interface("org.freedesktop.DBus.Properties")
        ->Get('org.freedesktop.DBus.Properties', $property);
}

method device_is_connected_via_a_supported_interface ($device) {
    assert_defined($device);

    my $iface = $self->get_device_property($device, 'DriveConnectionInterface');

    any { $_ eq $iface } (qw{sdio usb});
}

method device_is_optical ($device) {
    assert_defined($device);
    $self->get_device_property($device, 'DeviceIsOpticalDisc');
}

method partitions ($device) {
    assert_defined($device);

    my $partition_re;
    if ($device =~ m{mmcblk [0-9]+ \z}xms) {
        $partition_re = qr{\A$device[p][0-9]+\z};
    }
    else {
        $partition_re = qr{\A$device[0-9]+\z};
    }

    grep {
        $_ =~ m{$partition_re}xms
    } @{$self->udisks_object->EnumerateDevices()};
}

method device_partition_with_label ($device, $label) {
    assert_defined($device);
    assert_defined($label);

    first {
        defined && $label eq $self->get_device_property($_, 'PartitionLabel')
    } $self->partitions($device)
}

method device_has_partition_with_label ($device, $label) {
    assert_defined($device);
    assert_defined($label);

    defined $self->device_partition_with_label($device, $label);
}

method device_installed_with_tails_installer ($device) {
    assert_defined($device);

    'gpt' eq $self->get_device_property($device, 'PartitionTableScheme')
        or return;

    $self->device_has_partition_with_label($device, $self->system_partition_label)
        or return;

    return 1;
}

=head2 underlying_physical_device

Returns the physical block device UDI (e.g.
/org/freedesktop/UDisks/devices/sdb) on which the specified file
is stored.

=cut
method underlying_physical_device ($path) {
    assert_defined($path);

    my $st     = stat($path);
    my $device = $self->udisks_object->FindDeviceByMajorMinor(major($st->dev), minor($st->dev));
    my $parent = $self->get_device_property($device, 'PartitionSlave');
    if ($parent and $parent ne '/') {
        return $parent;
    }
    else {
        return $device;
    }
}

no Moose;
1; # End of Tails::UDisks
